@@ -16,11 +16,11 @@ CHI::Driver::SharedMem::t::CHIDriverTests
 
 =head1 VERSION
 
-Version 0.18
+Version 0.19
 
 =cut
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 
 =head1 SYNOPSIS
 
@@ -67,7 +67,7 @@ sub test_shmkey_required : Tests {
 		$cache = CHI->new(driver => 'SharedMem');
 	};
 	if($@) {
-		ok($@ =~ /CHI::Driver::SharedMem - no key given/);
+		ok($@ =~ /CHI::Driver::SharedMem - no shm_key given/);
 		ok(!defined($cache));
 	} else {
 		ok(0, 'Allowed shm_key to be undefined');
